{
  lib,
  stdenv,
  fetchurl,
  appimageTools,
  unzip,
  makeWrapper,
  # Notice: graphs will not sync without matching upstream's major electron version
  #         the specific electron version is set at top-level file to preserve override interface.
  #         whenever updating this package also sync electron version at top-level file.
  electron_27,
  autoPatchelfHook,
  git,
  nix-update-script,
  overrideSrc ? null,
}:
stdenv.mkDerivation (finalAttrs: let
  inherit (finalAttrs) pname version src appimageContents;
in {
  pname = "logseq";
  version = "0.10.9";

  src =
    if overrideSrc != null
    then overrideSrc
    else
      (fetchurl {
        url = "https://github.com/logseq/logseq/releases/download/${version}/logseq-linux-x64-${version}.AppImage";
        hash = "sha256-F3YbqgvL04P0nXaIVkJlCq/z8hUE0M0UutkBs2omuBe=";
        name = "${pname}-${version}.AppImage";
      });

  nativeBuildInputs =
    [makeWrapper]
    ++ lib.optionals stdenv.hostPlatform.isLinux [autoPatchelfHook]
    ++ lib.optionals stdenv.hostPlatform.isDarwin [unzip];
  buildInputs = [stdenv.cc.cc.lib];

  dontUnpack = stdenv.hostPlatform.isLinux;
  dontConfigure = true;
  dontBuild = true;

  installPhase =
    ''
      runHook preInstall
    ''
    + lib.optionalString stdenv.hostPlatform.isLinux (
      let
        appimageContents = appimageTools.extract {inherit pname src version;};
      in ''
        mkdir -p $out/bin $out/share/logseq $out/share/applications
        cp -a ${appimageContents}/{locales,resources} $out/share/logseq
        cp -a ${appimageContents}/Logseq.desktop $out/share/applications/logseq.desktop

        # remove the `git` in `dugite` because we want the `git` in `nixpkgs`
        chmod +w -R $out/share/logseq/resources/app/node_modules/dugite/git
        chmod +w $out/share/logseq/resources/app/node_modules/dugite
        rm -rf $out/share/logseq/resources/app/node_modules/dugite/git
        chmod -w $out/share/logseq/resources/app/node_modules/dugite

        mkdir -p $out/share/pixmaps
        ln -s $out/share/logseq/resources/app/icons/logseq.png $out/share/pixmaps/logseq.png

        substituteInPlace $out/share/applications/logseq.desktop \
          --replace Exec=Logseq Exec=logseq \
          --replace Icon=Logseq Icon=logseq
      ''
    )
    + lib.optionalString stdenv.hostPlatform.isDarwin ''
      mkdir -p $out/{Applications/Logseq.app,bin}
      cp -R . $out/Applications/Logseq.app
      makeWrapper $out/Applications/Logseq.app/Contents/MacOS/Logseq $out/bin/logseq
    ''
    + ''
      runHook postInstall
    '';

  postFixup = lib.optionalString stdenv.hostPlatform.isLinux ''
    # set the env "LOCAL_GIT_DIRECTORY" for dugite so that we can use the git in nixpkgs
    makeWrapper ${electron_27}/bin/electron $out/bin/logseq \
      --set "LOCAL_GIT_DIRECTORY" ${git} \
      --add-flags $out/share/logseq/resources/app \
      --add-flags "\''${NIXOS_OZONE_WL:+\''${WAYLAND_DISPLAY:+--ozone-platform-hint=auto --enable-features=WaylandWindowDecorations}}"
  '';

  passthru.updateScript = nix-update-script {};

  meta = {
    description = "Local-first, non-linear, outliner notebook for organizing and sharing your personal knowledge base";
    homepage = "https://github.com/logseq/logseq";
    changelog = "https://github.com/logseq/logseq/releases/tag/${version}";
    license = lib.licenses.agpl3Plus;
    sourceProvenance = with lib.sourceTypes; [binaryNativeCode];
    maintainers = with lib.maintainers; [cheeseecake];
    platforms = ["x86_64-linux" "aarch64-linux"] ++ lib.platforms.darwin;
    mainProgram = "logseq";
  };
})
